import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  area,
  curveBasis,
  max
} from 'd3';

//redering svg tag from html
const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

//Creating Area graph
const render = data => { 
  
  //Declaring constants we are using
  const title = 'US Auto Sales from 2005 to 2022';
  
  const xValue = d => d.Year; //display dates sur x axis
  const xAxisLabel = 'Year';

  const yValue = d => d.Sales;
  const yAxisLabel = 'Sales';
  
  const margin = { top: 60, right: 40, bottom: 88, left: 150 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  //container
  
  
  // Create X scale with year values
  const xScale = scaleTime()
  //complete set of values
    .domain(extent(data, xValue)) //Returns the minimum and maximum value in an array 
  																// from the given array using natural order.
    .range([0, innerWidth]) //Output bounds
    .nice(); //Round values out
  
  // Create Y scale with sales values
  const yScale = scaleLinear()
    .domain([0, max(data, yValue )])
    .range([innerHeight, 0])
    .nice();
  
  //center g in svg
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);//Center g in svg
  
  //creating X Axis  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(20);
  
  //creating Y Axis  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  //Appending Y axis label to svg
  const yAxisG = g.append('g').call(yAxis);
  	yAxisG.selectAll('.domain').remove();
  	yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -90)
      .attr('x', -innerHeight / 3)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .text(yAxisLabel);
  
  //Appending X axis label to svg
  const xAxisG = g.append('g').call(xAxis)
    	.attr('transform', `translate(0,${innerHeight})`);
  		xAxisG.select('.domain').remove();
  		xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 70)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  //Creating area graphs between x and y axis
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d))) //coordinates in pixels
  	.curve(curveBasis);
    
  //Appending area graph to svg
  g.append('path')
  	.attr('class', 'area_path')
  	.attr('d', areaGenerator(data))
  
  //Appending title of graph to svg
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);
};

//Loading data from URI
csv('https://gist.githubusercontent.com/ravitejabonam/420ad606e38a2ec1b233fb9a15ac8f2d/raw/747074a1250e5f1a2bfdcc3fef4349ac5c1eb247/US_TOTAL_AUTOMOTIVE_SALES_ANNUALLY.csv')
  .then(data => {
  	//console.log(data)
    data.forEach(d => {
    	d.GrowthRateYOY = +d.GrowthRateYOY;
      d.Year = new Date(d.Year)
    });
    render(data);
  });